import { ExceptionBase } from '@mobicoop/ddd-library';

export class CalendarTools {
  /**
   * Returns the first date corresponding to a week day (0 based monday)
   * within a date range
   */
  static firstDate = (
    weekDay: number,
    lowerDate: string,
    higherDate: string,
  ): Date => {
    if (weekDay < 0 || weekDay > 6)
      throw new CalendarToolsException(
        new Error('weekDay must be between 0 and 6'),
      );
    const lowerDateAsDate: Date = new Date(lowerDate);
    const higherDateAsDate: Date = new Date(higherDate);
    if (lowerDateAsDate.getDay() == weekDay) return lowerDateAsDate;
    const nextDate: Date = new Date(lowerDateAsDate);
    nextDate.setDate(
      lowerDateAsDate.getDate() + (7 - (lowerDateAsDate.getDay() - weekDay)),
    );
    if (lowerDateAsDate.getDay() < weekDay) {
      nextDate.setMonth(lowerDateAsDate.getMonth());
      nextDate.setFullYear(lowerDateAsDate.getFullYear());
      nextDate.setDate(
        lowerDateAsDate.getDate() + (weekDay - lowerDateAsDate.getDay()),
      );
    }
    if (nextDate <= higherDateAsDate) return nextDate;
    throw new CalendarToolsException(
      new Error('no available day for the given date range'),
    );
  };

  /**
   * Returns the last date corresponding to a week day (0 based monday)
   * within a date range
   */
  static lastDate = (
    weekDay: number,
    lowerDate: string,
    higherDate: string,
  ): Date => {
    if (weekDay < 0 || weekDay > 6)
      throw new CalendarToolsException(
        new Error('weekDay must be between 0 and 6'),
      );
    const lowerDateAsDate: Date = new Date(lowerDate);
    const higherDateAsDate: Date = new Date(higherDate);
    if (higherDateAsDate.getDay() == weekDay) return higherDateAsDate;
    const previousDate: Date = new Date(higherDateAsDate);
    previousDate.setDate(
      higherDateAsDate.getDate() - (higherDateAsDate.getDay() - weekDay),
    );
    if (higherDateAsDate.getDay() < weekDay) {
      previousDate.setMonth(higherDateAsDate.getMonth());
      previousDate.setFullYear(higherDateAsDate.getFullYear());
      previousDate.setDate(
        higherDateAsDate.getDate() -
          (7 + (higherDateAsDate.getDay() - weekDay)),
      );
    }
    if (previousDate >= lowerDateAsDate) return previousDate;
    throw new CalendarToolsException(
      new Error('no available day for the given date range'),
    );
  };
}

export class CalendarToolsException extends ExceptionBase {
  static readonly message = 'Calendar tools error';

  public readonly code = 'CALENDAR.TOOLS';

  constructor(cause?: Error, metadata?: unknown) {
    super(CalendarToolsException.message, cause, metadata);
  }
}
